;;; Written by: 
;;; This code will open all local #include files in a c or c++ file.
;;;


;; (defvar includes-path
;;   '("	.	" "./string-class" "./hh-test" "./string-class/klaus-test")
;;   "A list of directories which contains your include-files")

(defvar includes-path '(".")
  "A list of directories which contains your include-files")

(defmacro file-exists-and-is-not-a-dir (filename)
  "Return t if FILENAME exists and is not a directory."
  `(if (car (file-attributes ,filename))
       nil
     (file-exists-p ,filename)))

(defun get-all-include-files ()
"Returns all \"quoted\"-include-files in current-buffer if they exist in
`includes-path' as a list of filenames with full path-name. You can do
what you want with these filenames, e.g. open all files or something
similar.
Header-files are found if they are included as:
- #include \"header.h\"
- #include \"header.hh\"
- #include \"header\"

In the later case the .h-extension(or .hh-extension) is automaticaly added and header.h(or header.hh) is found.
Some code in this function is stolen from c-includes.el from John Wiegley."
(interactive)
  (let (all-includes)
    (save-excursion
      (goto-char (point-min))
      (while (not (eobp))
        (if (looking-at
             "^[ \t]*#[ \t]*include[ \t]*\"\\([^\"\n]+\\)\"")
            (let ((file (match-string-no-properties 1)))
              (if (not (file-exists-and-is-not-a-dir file))
                  (let ((dirs includes-path))
                    (setq dirs (cons "." dirs))
                    (while dirs
                      (let ((try (concat (car dirs) "/" file)))
                        (if (and (not (file-exists-and-is-not-a-dir try))
                                 (not (file-name-extension try))
                                 (file-exists-and-is-not-a-dir (concat try ".h")
							       ))
                            (setq file (concat try ".h")
                                  dirs nil)
                          (if (file-exists-and-is-not-a-dir try)
                              (setq file try
                                    dirs nil))))
                      (setq dirs (cdr dirs)))))
              (if (file-exists-and-is-not-a-dir file)
                  (setq all-includes (cons (expand-file-name file)
                                           all-includes)))))
        (forward-line)))
    all-includes))

(defun open-inc ()
  "Open all \"quoted\"-include-files in current-buffer."
  (interactive)
  (let ((all-incs (get-all-include-files)))
    (while all-incs
      (find-file (car all-incs))
      (setq all-incs (cdr all-incs)))))

(provide 'get-includes)

