

;;; Dired-mode. If a directory is on the current line, replace the current
;;; dired buffer with one containing the contents of the directory.
;;; Otherwise, invoke `dired-find-file' on the file. 
(defun dired-follow-file ()
  "In dired, visit the file or directory on this line.
If a directory is on the current line, replace the current
dired buffer with one containing the contents of the directory.
Otherwise, invoke `dired-find-file' on the file."
  (interactive)
  (let ((filename (dired-get-filename)))
    ;; if the file is a directory, replace the buffer with the
    ;;  directories contents
    (if (file-directory-p filename)
        (find-alternate-file filename)
      ;; otherwise simply perform a normal `dired-find-file'
      (dired-find-file))))

;;; Define some dired keys
(add-hook 'dired-mode-hook
	  (lambda ()
	    (local-set-key "\C-m" 'dired-follow-file)
	    (local-set-key "e" 'dired-follow-file)
	    (local-set-key "f" 'dired-follow-file)
	    (define-key dired-mode-map [left] 'dired-up-directory)
	    ;;(define-key dired-mode-map [right] 'dired-advertised-find-file)))
	    (define-key dired-mode-map [right] 'dired-follow-file)))
;;;

;;; Set dired-x buffer-local variables
(add-hook 'dired-mode-hook
  (function
   (lambda ()
     ;; Set dired-x buffer-local variables here.  For example:
     ;; (setq dired-omit-files-p t)
     (eval-after-load "dired"
       '(progn
	  ;; (defun my-dired-move-to-filename (&optional raise-error eol)
	  ;;   (or eol (setq eol (progn (end-of-line) (point))))
	  ;;   (beginning-of-line)
	  ;;   (forward-char 57)
	  ;;   (if (and (> (point) eol) raise-error)
	  ;;       (error "No file on this line")))
	  
	  ;; (defun my-dired-move-to-date (&optional raise-error eol)
	  ;;   (or eol (setq eol (progn (end-of-line) (point))))
	  ;;   (beginning-of-line)
	  ;;   (forward-char 44)
	  ;;   (if (and (> (point) eol) raise-error)
	  ;;       (error "No file on this line")))
	  
	  (defun my-dired-move-to-date (&optional raise-error eol)
	    (dired-move-to-filename raise-error eol)
	    (backward-char 13))
	  
	  ;; (defun my-dired-move-to-size (&optional raise-error eol)
	  ;;   (or eol (setq eol (progn (end-of-line) (point))))
	  ;;   (beginning-of-line)
	  ;;   (forward-char 31)
	  ;;   (if (and (> (point) eol) raise-error)
	  ;;       (error "No file on this line")))
	  
	  (defun my-dired-move-to-size (&optional raise-error eol)
	    (dired-move-to-filename raise-error eol)
	    (backward-char 25))
	  
	  (setq dired-font-lock-keywords
		(list
		 ;; Size
		 '("^. -"
		   (" *\\([0-9]+\\).*" (my-dired-move-to-size) nil
		    (1 font-lock-string-face)))
		 
		 ;; Date
		 '("^. [-dlcbs]"
		   ("\\(............\\).*" (my-dired-move-to-date) nil
		    (1 font-lock-type-face)))

		 ;; Directories
		 '("^. d"
		   (0 font-lock-warning-face)
		   (" *\\([0-9]+\\).*" (my-dired-move-to-size) nil
		    (1 font-lock-warning-face))
		   (".+" (dired-move-to-filename) nil
		    (0 font-lock-warning-face))
		   ("[#.][^.].*" (dired-move-to-filename) nil
		    (0 font-lock-warning-face))
		   )

		 ;; Omitted files
		 '(" [#.][^.].*$"            ; obtained from dired-omit-files
		   ("[#.][^.].*" (dired-move-to-filename) nil
		    (0 font-lock-builtin-face)))
		 
		 ;; Symbolic links
		 (list dired-re-sym          ; "^. [0-9 \t]*l"
		       '(0 font-lock-constant-face)
		       '(".+" (dired-move-to-filename) nil
			 (0 font-lock-constant-face)))
		 
		 ;; Backup and autosave files
		 '("[~%#]$"
		   (".+" (dired-move-to-filename) nil (0 font-lock-comment-face)))
		 '("\\.\\(bak\\)$"
		   (".+" (dired-move-to-filename) nil
		    (0 font-lock-comment-face)))
		 
		 ;; program files (source codes, any languages)
		 '("\\.\\(el\\|elc\\|p[ml]\\|cgi\\)$"
		   (".+" (dired-move-to-filename) nil
		    (0 font-lock-constant-face)))

		 ;; c/c++ source files
		 '("\\.\\(c\\|cc\\|cpp\\)$"
		   (".+" (dired-move-to-filename) nil
		    (0 font-lock-string-face)))

		 ;; c/c++ header files
		 '("\\.\\(h\\|hh\\|hpp\\)$"
		   (".+" (dired-move-to-filename) nil
		    (0 font-lock-variable-name-face)))
		 
		 ;; Zipped files
		 '("\\.\\(gz\\|tgz\\|bz2\\|Z\\|zip\\)$"
		   (".+" (dired-move-to-filename) nil (0 font-lock-type-face)))
		 
		 ;; World permissions
		 ;;'("^..[d-]......\\(r..\\|.w.\\|..[xs]\\) " 1 font-lock-type-face)
		 
		 ;; Write permissions
		 '("^. [d-]....\\(w\\)" 1 font-lock-type-face)
		 '("^. [d-]......\\(.w.\\)" 1 font-lock-type-face)
		 
		 ;; Directory name
		 (list dired-subdir-regexp   ; "^. \\([^\n]+\\)\\(:\\)[\n]"
		       '(1 font-lock-function-name-face)
		       '(1 'bold append))
		 
		 ;; Marked for deletion
		 '("^D"
		   (0 font-lock-warning-face prepend)
		   (".+" (dired-move-to-filename) nil
		    (0 font-lock-warning-face prepend))
		   (".+" (beginning-of-line) nil (0 'bold prepend)))
		 
		 ;; Marked (not for deletion)
		 '("^[^ D\n]"
		   (0 font-lock-warning-face prepend)
		   (".+" (beginning-of-line) nil (0 'bold prepend))))))))))
;;;

(provide 'my-dired)
